/* LEDDefinition.cpp */

#include "LEDDefinition.h"

LEDColor offColor = { name: "off", red: 0, green: 0, blue: 0};
LEDColor dimmedRedColor = { name: "dimmedRed", red: 80, green: 0, blue: 0};

#define MAX_COLORS 20
static LEDColor ledColor[MAX_COLORS];
static int numberOfColors = 0;

LEDColor *LEDColor_get(char *name) {
	for (int i=0; i<numberOfColors; ++i) {
		if (strcmp(name, ledColor[i].name) == 0)
			return &ledColor[i];
	}
	LOG(LL_ERROR, ("Error: Color not found: %s", name));
	return &offColor;
}

void LEDColor_add(char *name, uint8_t red, uint8_t green, uint8_t blue) {
	if (numberOfColors < MAX_COLORS) {
		ledColor[numberOfColors].name = name;
		ledColor[numberOfColors].red = red;
		ledColor[numberOfColors].green = green;
		ledColor[numberOfColors].blue = blue;
		++numberOfColors;
	} else {
		LOG(LL_ERROR, ("Too many color definitions, igrnoring %s (%d, %d, %d)", name, red, green, blue));
	}
}

void LEDColor_init(bool loadDefaultData) {
	if (loadDefaultData) {
		LEDColor_add("off", 0, 0, 0);
		LEDColor_add("red", 255, 0, 0);
		LEDColor_add("green", 0, 255, 0);
		LEDColor_add("blue", 0, 0, 255);
		LEDColor_add("brightWhite", 255, 255, 255);
		LEDColor_add("white", 180, 180, 180);
		LEDColor_add("dimmedWhite", 100, 100, 100);
		LEDColor_add("dimmedRed", 100, 0, 0);
	}
}

LEDDefinition ledDefinition[MAX_LEDS];
static int numberOfLEDDefinitions = 0;

static int verifyLedDefinitionNum(int ledNum) {
	if (ledNum < 0) {
		LOG(LL_ERROR, ("Error: verifyLedDefinitionNum failed for ledNum=%d", ledNum));
		return 0;
	}
	return ledNum % numberOfLEDDefinitions;
}

LEDDefinition *LEDDefinition_get(int ledNum) {
	ledNum = verifyLedDefinitionNum(ledNum);
	return &ledDefinition[ledNum];
}

void LEDDefinition_add(char *level, char *room, char *id, uint8_t red, uint8_t green, uint8_t blue) {
	if (numberOfLEDDefinitions < MAX_LEDS) {
		ledDefinition[numberOfLEDDefinitions].level = level;
		ledDefinition[numberOfLEDDefinitions].room = room;
		ledDefinition[numberOfLEDDefinitions].id = id;
		ledDefinition[numberOfLEDDefinitions].onColor.name = "autoColor";
		ledDefinition[numberOfLEDDefinitions].onColor.red = red;
		ledDefinition[numberOfLEDDefinitions].onColor.green = green;
		ledDefinition[numberOfLEDDefinitions].onColor.blue = blue;
		++numberOfLEDDefinitions;
	} else {
		LOG(LL_ERROR, ("Error: Too many LED Definitions to load, ignoring level=%s, room=%s, id=%s", level, room, id));
	}
}

void LEDDefinition_init(int numberOfLeds, bool loadDefaultData) {
	// initialize
	numberOfLEDDefinitions = 0;
	if (loadDefaultData) {
		do {
			LEDDefinition_add("EG", "Wohnzimmer", "EG-WoZi", 200,200,200);
			LEDDefinition_add("EG", "Bad", "EG-Bad", 130,130,130);
			LEDDefinition_add("EG", "Schlafzimmer", "EG-Schlafen", 150,150,130);
			LEDDefinition_add("EG", "Buero 0.1", "EG-Buero-1", 150,130,130);
			LEDDefinition_add("EG", "Buero 0.2", "EG-Buero-2", 150,200,200);
			LEDDefinition_add("EG", "Flur", "EG-Flur-1", 100,100,100);
			LEDDefinition_add("EG", "Flur", "EG-Flur-2", 140,100,100);
			LEDDefinition_add("OG1", "Buero 1.1", "OG1-WoZi", 200,200,200);
			LEDDefinition_add("OG1", "Bad OG1", "OG1-Bad", 130,130,130);
			LEDDefinition_add("OG1", "Buero 1.2", "OG1-Schlafen", 150,150,130);
			LEDDefinition_add("OG1", "Buero 1.3", "OG1Buero-1", 150,150,130);
			LEDDefinition_add("OG1", "Buero 1.4", "OG1-Buero-2", 150,200,200);
			LEDDefinition_add("OG1", "Flur", "OG1-Flur-1", 100,100,100);
			LEDDefinition_add("OG1", "Flur", "OG1-Flur-2", 140,100,100);
		} while(numberOfLEDDefinitions < numberOfLeds);
	}
}

char *LEDDefinition_getLevel(int ledNum) {
	ledNum = verifyLedDefinitionNum(ledNum);
	return ledDefinition[ledNum].level;
}
char *LEDDefinition_getRoom(int ledNum) {
	ledNum = verifyLedDefinitionNum(ledNum);
	return ledDefinition[ledNum].room;
}
char *LEDDefinition_getId(int ledNum) {
	ledNum = verifyLedDefinitionNum(ledNum);
	return ledDefinition[ledNum].id;
}
int LEDDefinition_getOnColorRed(int ledNum) {
	ledNum = verifyLedDefinitionNum(ledNum);
	return ledDefinition[ledNum].onColor.red;
}
int LEDDefinition_getOnColorGreen(int ledNum) {
	ledNum = verifyLedDefinitionNum(ledNum);
	return ledDefinition[ledNum].onColor.green;
}
int LEDDefinition_getOnColorBlue(int ledNum) {
	ledNum = verifyLedDefinitionNum(ledNum);
	return ledDefinition[ledNum].onColor.blue;
}