#include <stdio.h>

#include "common/platform.h"
#include "common/cs_file.h"
#include "mgos_app.h"
#include "mgos_gpio.h"
#include "mgos_sys_config.h"
#include "mgos_system.h"
#include "mgos_timers.h"
#include "fw/src/mgos_hal.h"
#include "fw/src/mgos_dlsym.h"
#include "mjs.h"
#include "LEDDefinition.h"
#include "AnimationConfig.h"
#include "LEDState.h"

#if CS_PLATFORM == CS_P_ESP8266
#define LED_GPIO 2 /* On ESP-12E there is a blue LED connected to GPIO2  */
#elif CS_PLATFORM == CS_P_ESP32
#define LED_GPIO 21 /* No LED on DevKitC, use random GPIO close to GND pin */
#elif CS_PLATFORM == CS_P_CC3200
#define LED_GPIO 64 /* The red LED on LAUNCHXL */
#elif(CS_PLATFORM == CS_P_STM32) && defined(BSP_NUCLEO_F746ZG)
/* Nucleo-144 F746 */
#define LED_GPIO STM32_PIN_PB7 /* Blue LED */
#elif(CS_PLATFORM == CS_P_STM32) && defined(BSP_DISCO_F746G)
/* Discovery-0 F746 */
#define LED_GPIO STM32_PIN_PI1 /* Green LED */
#else
#error Unknown platform
#endif

int get_led_gpio_pin(void) {
	return LED_GPIO;
}

LEDStateEngine ledStateEngine;
static bool stateEngineRunning = false;

static void stateEngineTickTimer() {
	if (stateEngineRunning) {
		// LOG(LL_INFO, ("digg"));
		LEDStateEngine_tick();
	} else {
		LOG(LL_INFO, ("--paused--"));
	}
}

static void delayed_boot() {
	LOG(LL_INFO, ("*** Start timer for LED state engine ticks"));
	mgos_set_timer(100, true, stateEngineTickTimer, NULL); // every 0.1 second call timer_cb
}

enum mgos_app_init_result mgos_app_init(void) {
	int numberOfLeds = mgos_sys_config_get_led_count();
	bool loadDefaultData = mgos_sys_config_get_led_useDefaults();

	LEDColor_init(loadDefaultData);
	LEDDefinition_init(numberOfLeds, loadDefaultData);
	AnimationConfig_init(loadDefaultData);
	LEDStateEngine_init(numberOfLeds);
	if (loadDefaultData) {
		stateEngineRunning = true;
	}

	mgos_set_timer(5000, false, delayed_boot, NULL); // after 5 seconds boot the functions
	return MGOS_APP_INIT_SUCCESS;
}

// javascript adapter functions:
void addColor(char *name, int red, int green, int blue) {
	LEDColor_add(name, red, green, blue);
}
void addLedDefinition(char *level, char *room, char *id, int red, int green, int blue) {
	LEDDefinition_add(level, room, id, red, green, blue);
}
void addAnimationStep(int ledIndex, int ledMode, int duration) {
	LEDMode m = LEDMode_off;
	switch (ledMode) {
		case 1: m = LEDMode_on; break;
		case 2: m = LEDMode_off; break;
		case 3: m = LEDMode_blink; break;
		case 4: m = LEDMode_tv; break;
		case 5: m = LEDMode_fire; break;
		default: m = LEDMode_off;
	}
	AnimationStep_add(ledIndex, m, duration);
}

void startLEDStateEngine(void) {
	stateEngineRunning = true;
	LOG(LL_INFO, ("LEDStateEngine started"));
}
void pauseLEDStateEngine(void) {
	stateEngineRunning = false;
	LOG(LL_INFO, ("LEDStateEngine stopped"));
}
